function [log_info] = synthetic_data(files_param, rul_param, rul_typ )
    % SYNTHETIC_BOOLEAN_DATA Generates synthetic boolean data for redescription
    % mining experiments with given parameters.
    % EXAMPLE PARAMETERS:
    % -------------------
    %	files_param = struct(	'directory_out', './data/', ...			% output directory
    %				'suffix','synthe', ...				% filename suffix
    %				'differents', 10, ...				% number of different matrix pairs for each series
    %				'disp', true, ...				% display generated data
    %				'log_to_file', true, ...			% write generation information to log file
    %				'dat_ext_L', '.datbool', ...			% file extension left hand side matrix
    %				'dat_ext_R', '.datbool', ...			% file extension right hand side matrix
    %				'info_ext', '.info');				% file extension log file
    % 
    %	rul_param = struct( 	'nb_rows', {100}, ... 				% total nb of rows
    %				'nb_cols_L', {20}, ... 				% number of columns of the left hand side matrix
    %				'nb_cols_R', {20}, ... 				% number of columns of the right hand side matrix
    %				'supp_rows_L', {25}, ... 			% number of supporting rows of the left hand side matrix
    %				'supp_rows_R', {25}, ... 			% number of supporting rows of the right hand side matrix
    %				'nb_variables_L', {5}, ... 			% number of supporting variables of the left hand side matrix
    %				'nb_variables_R', {5}, ... 			% number of supporting variables of the right hand side matrix
    %				'contrib', {4}, ... 				% contribution
    %				'offset', {0}, ... 				% offset before support of right hand side matrix
    %				'preserving', {true} , ... 			% preserve the original support of the rules when adding noise (boolean)
    %				'margin_L', {1}, ... 				% margin left 1=boolean data
    %				'margin_R', {1}, ... 				% margin right 1=boolean data
    %				'density', {0.01}, ... 				% noise density
    %				'density_blurr_OR', {0.8}, ... 			% supporting columns blurr OR
    %				'density_blurr_AND', {0.2} ); 			% supporting columns blurr AND
    % 
    %	rul_typ = struct(	'OR_L', {true, true, false, false}, ... 	% type of rule Left Hand Side
    %				'OR_R', {true, false, true, false}, ... 	% type of rule Right Hand Side 
    %				'label', {'OO', 'OA', 'AO', 'AA'});     	% prefixes of the corresponding files
    
    
    titles_str = {'p:', 'd:', 'bd:', 'br'};
    if files_param.log_to_file
        log_info = [files_param.directory_out files_param.suffix  files_param.info_ext];
        fid = fopen(log_info,'w');
    else
        log_info = '-';
        fid = 1; % stdout
        
    end    
    %fprintf(fid, strrep('##basename\t i(sett_id) l(conf_id) j(copy_id) acc nb_variables_L nb_variables_R OR_L OR_R offset supp_rows_L supp_rows_R preserving margin_L margin_R density density_blurr_OR density_blurr_AND\n', ' ', '\t'));
    for i = 1:length(rul_param)
        titles_fig = [rul_param(i).preserving rul_param(i).density rul_param(i).density_blurr_OR rul_param(i).density_blurr_AND];
        for l = 1:length(rul_typ)
            for j = 0:files_param.differents-1
                [L, R, acc] = synthetic_boolean_data_pair(rul_param(i), rul_typ(l));
                format_setts = strrep([num2str(i) rul_typ(l).label '-' num2str(j) ' %i %i %i %f %i %i %i %i %i %i %i %i %f %f %f %f %f\n'], ' ', '\t');
                to_log_setts = [i, l, j, acc, rul_param(i).nb_variables_L, rul_param(i).nb_variables_R, rul_typ(l).OR_L, rul_typ(l).OR_R, rul_param(i).offset, rul_param(i).supp_rows_L, rul_param(i).supp_rows_R, rul_param(i).preserving,  rul_param(i).margin_L, rul_param(i).margin_R, rul_param(i).density, rul_param(i).density_blurr_OR, rul_param(i).density_blurr_AND];
                if length(files_param.directory_out) > 0
                    save_matrix(L, [files_param.directory_out 'L' num2str(i) rul_typ(l).label '-' num2str(j) '_' files_param.suffix], files_param.dat_ext_L);
                    save_matrix(R, [files_param.directory_out 'R' num2str(i) rul_typ(l).label '-' num2str(j) '_' files_param.suffix], files_param.dat_ext_R);
                end
                fprintf(fid, format_setts, to_log_setts);
            end
            if files_param.disp
                figure(i);
                subplot(1,2,1); imagesc(L); subplot(1,2,2); imagesc(R); colormap(gray); %colorbar()
                title([titles_str{l} num2str(titles_fig(l)) ', acc:' num2str(acc)])
            end
        end
    end
    if files_param.log_to_file
        fclose(fid);
    end   

function [L, R, acc] = synthetic_boolean_data_pair( rul_param, rul_typ )
    %SYNTHETIC_BOOLEAN_DATA Generates a pair of synthetic boolean matrices for redescription
    %mining experiments.
    
    
    %% skeleton    
    if rul_typ.OR_L
        [ NL, ML, QL ] = makeRuleSkeleton(rul_param.nb_variables_L,  rul_param.supp_rows_L-rul_param.contrib*rul_param.nb_variables_L, rul_param.nb_rows-rul_param.supp_rows_L, rul_param.contrib, rul_param.density_blurr_OR, false);
    else
        [ NL, QL, ML ] = makeRuleSkeleton(rul_param.nb_variables_L,  rul_param.nb_rows-rul_param.supp_rows_L-rul_param.contrib*rul_param.nb_variables_L, rul_param.supp_rows_L, rul_param.contrib, rul_param.density_blurr_AND, true);
    end

    if rul_typ.OR_R
        [ NR, MR, QR ] = makeRuleSkeleton(rul_param.nb_variables_R,  rul_param.supp_rows_R-rul_param.contrib*rul_param.nb_variables_R, rul_param.nb_rows-rul_param.supp_rows_R, rul_param.contrib, rul_param.density_blurr_OR, false);
    else
        [ NR, QR, MR ] = makeRuleSkeleton(rul_param.nb_variables_R,  rul_param.nb_rows-rul_param.supp_rows_R-rul_param.contrib*rul_param.nb_variables_R, rul_param.supp_rows_R, rul_param.contrib, rul_param.density_blurr_AND, true);
    end

    %% compose the matrices
    if rul_typ.OR_L
        if rul_typ.OR_R
            rulec_L = [ML; NL; QL];
            rulec_R = [QR([1:rul_param.offset],:); NR(:,end:-1:1); MR(:,end:-1:1); QR(1 + rul_param.offset:end,:)];
        else
            rulec_L = [QL([1:rul_param.offset],:); NL; ML; QL(1 + rul_param.offset:end,:)];
            rulec_R = [MR; QR; NR];
        end
    else
        if rul_typ.OR_R
            rulec_L = [ML; QL; NL];
            rulec_R = [QR([1:rul_param.offset],:); NR; MR; QR(1 + rul_param.offset:end,:)];
        else
            if rul_param.contrib*rul_param.nb_variables_L > size(QR,1)
                error('QR too small !')
            end
            rulec_L = [ML; QL(:,end:-1:1); NL];
            rulec_R = [QR(rul_param.contrib*rul_param.nb_variables_L + [1:rul_param.offset],:); MR; QR(rul_param.contrib*rul_param.nb_variables_L + 1 + rul_param.offset:end,:); NR; QR(1:rul_param.contrib*rul_param.nb_variables_L,:)];
        end

    end
    

    n = full(sprand(rul_param.nb_rows, rul_param.nb_cols_L+rul_param.nb_cols_R,rul_param.density) > 0); 
    if not(mod(rul_param.preserving,2) ) 
        rulec_L = bitxor(rulec_L, n(:, 1:rul_param.nb_variables_L));
        rulec_R = bitxor(rulec_R, n(:, rul_param.nb_cols_L+1:rul_param.nb_cols_L+rul_param.nb_variables_R));
    end
    L = [rulec_L n(:, rul_param.nb_variables_L+1:rul_param.nb_cols_L)];
    R = [rulec_R n(:, rul_param.nb_cols_L+rul_param.nb_variables_R+1:end)];
    if rul_param.preserving > 1
        idsEL = find(sum(L,2)==0);
        colsAddL = randi(rul_param.nb_cols_L-rul_param.nb_variables_L, [length(idsEL), 1]);
        maskL = sparse(idsEL,colsAddL,ones(size(colsAddL)),rul_param.nb_rows, rul_param.nb_cols_L-rul_param.nb_variables_L);
        L(:, rul_param.nb_variables_L+1:rul_param.nb_cols_L) = L(:, rul_param.nb_variables_L+1:rul_param.nb_cols_L) + maskL;
        idsER = find(sum(R,2)==0);
        colsAddR = randi(rul_param.nb_cols_R-rul_param.nb_variables_R, [length(idsER), 1]);
        maskR = sparse(idsER,colsAddR,ones(size(colsAddR)),rul_param.nb_rows, rul_param.nb_cols_R-rul_param.nb_variables_R);
        R(:, rul_param.nb_variables_R+1:rul_param.nb_cols_R) = R(:, rul_param.nb_variables_R+1:rul_param.nb_cols_L) + maskR;
    end
    

    prec_L=100;
    prec_R=100;

    acc = computeAcc(L, rul_param.nb_variables_L, rul_typ.OR_L, R, rul_param.nb_variables_R, rul_typ.OR_R);
    if rul_param.margin_L < 1
        L =round(prec_L*abs(L-((1-max(rul_param.margin_L, eps))/2*rand(size(L)))))/prec_L;
    end
    if rul_param.margin_R < 1
        R =round(prec_R*abs(R-((1-max(rul_param.margin_R, eps))/2*rand(size(R)))))/prec_R;  
    end

function acc = computeAcc(L, nb_variables_L, opOR_L, R, nb_variables_R, opOR_R)
    suppL = computeSuppVect(L, nb_variables_L, opOR_L);
    suppR = computeSuppVect(R, nb_variables_R, opOR_R);
    inter_c = sum(suppL & suppR);
    union_c = sum(suppL | suppR);
    acc = inter_c(1,1)/union_c(1,1);
        
function suppV = computeSuppVect(M, nb_variables, opOR)
    suppV = M(:,1)>0;
    if  opOR
        for i = 2:nb_variables
            suppV = suppV | M(:,i)>0;
        end
    else
        for i = 2:nb_variables
            suppV = suppV & M(:,i)>0;
        end
    end

function [ N, M, Q ] = makeRuleSkeleton(rows_N,  rows_M, rows_Q, c, density_add, invert)    

    if rows_N == 0 
        N = [];
        M = zeros(rows_M,0);
        Q = zeros(rows_Q,0);
    else
        N = repmat(eye(rows_N),c,1);
        diags = repmat(eye(rows_N), ceil(rows_M/rows_N),1);
        ids = randperm(rows_M*rows_N);
        nb_vals = round(rows_M*rows_N*density_add); %  max(0,min(1,normrnd(density_add, 0.05))));
        mask = zeros(rows_M*rows_N,1);
        mask(ids(1:nb_vals)) = 1;
        M = diags(randperm(rows_M),:) | reshape(mask, rows_M, rows_N);
        Q = zeros(rows_Q, rows_N);
        if invert
            N = 1-N;
            M = 1-M;
            Q = 1-Q;
        end
    end

##
##    Feb, 2011. EG. ReReMi 1.0
##
